VERSION 5.00
Begin {C62A69F0-16DC-11CE-9E98-00AA00574A4F} frmIterateRandom 
   Caption         =   "Iterative Random Point Generator"
   ClientHeight    =   4188
   ClientLeft      =   48
   ClientTop       =   336
   ClientWidth     =   4680
   OleObjectBlob   =   "frmIterateRandom.frx":0000
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "frmIterateRandom"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False



Private Sub cmdRun_Click()
''
''  (c) Version May 15 2002 Dr. M. Sawada 2002 msawada@uottawa.ca
''  This program creates a set of n random points within a given polygon (single part)
''  NOT for every polygon but only a single selected polygon or a theme containing one polygon.
''  The program gets the envelope of the polygon and takes a random x and y value
''  between min and max x and min and max y.
''  The random coordinate is of a double type (real number) not an integer.
''
''--------------- MODIFICATIONS --------------
''  Modified Sept 28 2005 charlene.nielsen@ualberta.ca, Biological Sciences
''  Code was added to allow iterative generation of random point files - i.e. for statistical testing.
''  The Iteration, X_Coord and YCoord of the each random point is added to the attribute table.
''  Note: Output will be saved to the same directory that the map document resides in!!!
''
''--------------- INSTRUCTIONS --------------
''  Associate (load) this form with a UIButtonControl in ArcMap.
''  use the code
''      ''
''      load frmRandom
''      frmRandom.show
''-------------------------------------------
    
' Check to see if user input is valid
    If Not (IsNumeric(txtIterations.Text) And IsNumeric(txtNumSamples.Text)) Then
        MsgBox "One of your inputs is not numeric; either the number of samples, or number of iterations.", , "random"
        Exit Sub
    End If
    
' Usual set up
Dim theIMXDoc As IMxDocument
Set theIMXDoc = ThisDocument
Dim theImap As IMap
Set theImap = theIMXDoc.FocusMap
Dim theilayer As ILayer
Set theilayer = theIMXDoc.SelectedLayer

' Get the spatial reference
    Dim pSR As ISpatialReference
    Set pSR = theImap.SpatialReference

' Check to see if a layer is selected or not
    If theIMXDoc.SelectedItem Is Nothing Then
      MsgBox "Select a feature layer in the table of contents as the input feature class."
      Exit Sub
    End If
    
    If Not TypeOf theIMXDoc.SelectedItem Is IFeatureLayer Then
      MsgBox "No feature layer selected. Select a feature layer."
      Exit Sub
    End If

    Dim theIFeatureLayer As IFeatureLayer
    Set theIFeatureLayer = theilayer
    
' Get the layer name
    Dim theilayername As String
    theilayername = theilayer.Name

    Dim theIGeometry As IGeometry
    
' Get all records or selected records and then get the first one in theifeature
    Dim theIFeatureSelection As IFeatureSelection
    Set theIFeatureSelection = theIFeatureLayer
    Dim theISelectionSet As ISelectionSet
    Set theISelectionSet = theIFeatureSelection.SelectionSet
    Dim theIFeatureCursor As IFeatureCursor
    Dim theIFeatureClass As IFeatureClass
        If chkSelected.Value Then
            theISelectionSet.Search Nothing, False, theIFeatureCursor
            If theISelectionSet.Count > 1 Then 'only one polygon can be selected
                MsgBox "You can select at most one polygon. Close and select a single polygon.", , "Random Points"
                Exit Sub
            ElseIf theISelectionSet.Count = 0 Then
                Set theIFeatureCursor = theIFeatureLayer.Search(Nothing, False)
                Set theIFeatureClass = theIFeatureLayer.FeatureClass
                    If theIFeatureClass.FeatureCount(Nothing) > 1 Then
                            MsgBox "You have more than one polygon in your currently selected layer.  This program only creates a random set of points in a single polygon.  Close this program and select a single polygon in your layer using the 'Select Features' tool on the toolbar. OR You have chosen a polygon but not clicked on the 'Use selected polygon' checkbox.", , "Random Points"
                        Exit Sub
                    End If
            End If
        Else
            Set theIFeatureCursor = theIFeatureLayer.Search(Nothing, False)
            Set theIFeatureClass = theIFeatureLayer.FeatureClass
            If theIFeatureClass.FeatureCount(Nothing) > 1 Then
                    MsgBox "You have more than one polygon in your currently selected layer.  This program only creates a random set of points in a single polygon.  Close this program and select a single polygon in your layer using the 'Select Features' tool on the toolbar. OR You have chosen a polygon but not clicked on the 'Use selected polygon' checkbox.", , "Random Points"
                Exit Sub
            End If
        End If

' Set theifeaturecursor = theifeaturelayer.Search(Nothing, False)
    Dim theifeature As IFeature
    Set theifeature = theIFeatureCursor.NextFeature
    Set theIGeometry = theifeature.Shape
    theIGeometry.Project pSR
        ' Check to see if a polygon feature layer is selected
        If Not (theIGeometry.GeometryType = esriGeometryPolygon) Then
            MsgBox "You need to select a polygon layer for this program.  Close and select a polygon layer.", , "Random"
            Exit Sub
        End If

' Set the loop counter for iterations
Dim itCounter As Long
itCounter = 1

' Loop through each iteration until txtIterations value is met
Do While itCounter <= txtIterations

Dim theIPolygon As IPolygon
Set theIPolygon = theIGeometry

Dim theIEnvelope As IEnvelope
Set theIEnvelope = theIGeometry.Envelope

sett = CLng(txtNumSamples.Text)
Dim theIPoint() As IPoint
ReDim theIPoint(0 To sett) As IPoint
'Set theipoint = New Point

Dim theirel As IRelationalOperator
Set theirel = New Polygon
Set theirel = theIPolygon

' Set up multipoint for constructing convex hull and/or rectangle
    Dim theIMultiPoint As IMultipoint
    Set theIMultiPoint = New Multipoint
    Dim thePointCollection As IPointCollection
    Set thePointCollection = theIMultiPoint

    upperboundx = theIEnvelope.xMax
    lowerboundx = theIEnvelope.xMin
    upperboundy = theIEnvelope.yMax
    lowerboundy = theIEnvelope.yMin
    
counter = 0
Dim theipointtest As IPoint
Set theipointtest = New Point
Randomize (CDbl(Now()) + Timer)
Do While counter < sett
xval = (upperboundx - lowerboundx) * Rnd + lowerboundx
yval = (upperboundy - lowerboundy) * Rnd + lowerboundy
theipointtest.PutCoords xval, yval
    If theirel.Contains(theipointtest) Then
        Set theIPoint(counter) = New Point
        theIPoint(counter).PutCoords xval, yval
        counter = counter + 1
    End If
Loop

thePointCollection.AddPoints sett, theIPoint(0)

' Call the make polygon shapefile function
    Call MakeShapefile(thePointCollection, theilayername, pSR, itCounter)

' Increment the loop counter
itCounter = itCounter + 1
Loop

End Sub

Public Function MakeShapefile(thePolyGeometry As IGeometry, theilayername, theSpatialRef As ISpatialReference, itCounter)
On Error GoTo EH

  Dim strfolder As String
  Dim strname As String ' Don't include .shp extension
  Const strShapeFieldName As String = "Shape"
  
    strfolder = CurDir ' Set as current directory
    strname = "random_" & theilayername & itCounter
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
  Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  Set pFWS = pWorkspaceFactory.OpenFromFile(strfolder, 0)

  ' Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
  Set pFields = New Fields
  Set pFieldsEdit = pFields
  
  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' It will need a geometry definition, with a spatial reference
  Set pField = New Field
  Set pFieldEdit = pField
  pFieldEdit.Name = strShapeFieldName
  pFieldEdit.Type = esriFieldTypeGeometry
  
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
  Set pGeomDef = New GeometryDef
  Set pGeomDefEdit = pGeomDef
  With pGeomDefEdit
    .GeometryType = esriGeometryPoint
    Set .SpatialReference = theSpatialRef 'New UnknownCoordinateSystem 'pSR '
  End With
  Set pFieldEdit.GeometryDef = pGeomDef
  pFieldsEdit.AddField pField

  ' Add new Iteration field
  Set pField = New Field
  Set pFieldEdit = pField
  With pFieldEdit
      .Type = esriFieldTypeLong
      .Name = "Iteration"
      .Precision = 6
  End With
  pFieldsEdit.AddField pField

  ' Add new X field
  Set pField = New Field
  Set pFieldEdit = pField
  With pFieldEdit
      .Type = esriFieldTypeDouble
      .Name = "X_Coord"
      .Precision = 20
      .Scale = 4
  End With
  pFieldsEdit.AddField pField

  ' Add new Y field
  Set pField = New Field
  Set pFieldEdit = pField
  With pFieldEdit
      .Type = esriFieldTypeDouble
      .Name = "Y_Coord"
      .Precision = 20
      .Scale = 4
  End With
  pFieldsEdit.AddField pField

  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim theIFeatureClass As IFeatureClass
  Set theIFeatureClass = pFWS.CreateFeatureClass(strname, pFields, Nothing, _
                                           Nothing, esriFTSimple, strShapeFieldName, "")

' Populate the fields
        ' Set a pointcollection from the igeometry
        Dim theIPC As IPointCollection
        Set theIPC = thePolyGeometry
        Dim theIEnumVertex As IEnumVertex
        Set theIEnumVertex = theIPC.EnumVertices
        Dim theIPoint As IPoint
        Dim pindex As Long, vertindex As Long
        theIEnumVertex.Next theIPoint, pindex, vertindex

        Dim theIFeatureCursor As IFeatureCursor
        Dim theIFeatureBuffer As IFeatureBuffer
        ' Open the feature cursor and feature buffer
        Set theIFeatureCursor = theIFeatureClass.Insert(True)
        Set theIFeatureBuffer = theIFeatureClass.CreateFeatureBuffer
        ' Get the list of fields
        Dim theIterationFieldIndex As Long
        Dim theX_CoordFieldIndex As Long
        Dim theY_CoordFieldIndex As Long
        Dim theIFields As IFields
        Set theIFields = theIFeatureClass.Fields
        theShapeIndex = theIFields.FindField("shape")
        theIterationFieldIndex = theIFields.FindField("Iteration")
        theX_CoordFieldIndex = theIFields.FindField("X_Coord")
        theY_CoordFieldIndex = theIFields.FindField("Y_Coord")

        ' Loop through each point and populate fields
        Do
                theIFeatureBuffer.Value(theShapeIndex) = theIPoint
                theIFeatureBuffer.Value(theIterationFieldIndex) = itCounter
                theIFeatureBuffer.Value(theX_CoordFieldIndex) = theIPoint.X
                theIFeatureBuffer.Value(theY_CoordFieldIndex) = theIPoint.Y
                theIFeatureCursor.InsertFeature theIFeatureBuffer
                theIEnumVertex.Next theIPoint, pindex, vertindex
        Loop Until theIPoint Is Nothing
                
        Call AddShapeFile(strfolder, strname)
        
    Exit Function
EH:
    MsgBox Err.Description, vbInformation, "createShapefile"
End Function

Private Function AddShapeFile(strfolder As String, strname As String)
  Dim pWorkspaceFactory As IWorkspaceFactory
  Dim pFeatureWorkspace As IFeatureWorkspace
  Dim pFeatureLayer As IFeatureLayer
  Dim pMxDocument As IMxDocument
  Dim pMap As IMap
  'Create a new ShapefileWorkspaceFactory object and open a shapefile folder
  Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  Set pFeatureWorkspace = pWorkspaceFactory.OpenFromFile(strfolder, 0)
  'Create a new FeatureLayer and assign a shapefile to it
  Set pFeatureLayer = New FeatureLayer
  Set pFeatureLayer.FeatureClass = pFeatureWorkspace.OpenFeatureClass(strname)
  pFeatureLayer.Name = pFeatureLayer.FeatureClass.AliasName
  'Add the FeatureLayer to the focus map
  Set pMxDocument = Application.Document
  Set pMap = pMxDocument.FocusMap
  pMap.AddLayer pFeatureLayer
End Function

Private Sub cmdClose_Click()
Unload frmIterateRandom
 frmIterateRandom.Hide
End Sub

